/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMHTMLElement.idl"

/**
 * The nsIDOMHTMLFieldSetElement interface is the interface to a
 * [X]HTML fieldset element.
 *
 * This interface is trying to follow the DOM Level 2 HTML specification:
 * http://www.w3.org/TR/DOM-Level-2-HTML/
 *
 * with changes from the work-in-progress WHATWG HTML specification:
 * http://www.whatwg.org/specs/web-apps/current-work/
 */

interface nsIDOMValidityState;

[scriptable, uuid(eb09f2e9-f0fc-4000-ab15-e68b0ac09bbd)]
interface nsIDOMHTMLFieldSetElement : nsIDOMHTMLElement
{
           attribute boolean                disabled;
  readonly attribute nsIDOMHTMLFormElement  form;
           attribute DOMString              name;

  readonly attribute DOMString              type;

  readonly attribute nsIDOMHTMLCollection   elements;

  // The following lines are parte of the constraint validation API, see:
  // http://www.whatwg.org/specs/web-apps/current-work/#the-constraint-validation-api
  readonly attribute boolean               willValidate;
  readonly attribute nsIDOMValidityState   validity;
  readonly attribute DOMString             validationMessage;
  boolean checkValidity();
  void setCustomValidity(in DOMString error);
};
