/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim: set ts=2 et sw=2 tw=80: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIURI;

[scriptable, function, uuid(7b0f9928-0ddc-42c7-b9f2-6b2308b90b18)]
interface nsIUsageCallback : nsISupports
{
  void
  onUsageResult(in nsIURI aURI,
                in unsigned long long aUsage,
                in unsigned long long aFileUsage,
                in unsigned long aAppId,
                in boolean aInMozBrowserOnly);
};
