/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIDOMWindow;

[scriptable, uuid(731C6C50-67D6-11d4-9529-0020183BF181)]
interface nsICommandHandlerInit : nsISupports
{
	attribute nsIDOMWindow window;
};

[scriptable, uuid(34A4FCF0-66FC-11d4-9528-0020183BF181)]
interface nsICommandHandler : nsISupports
{
	/*
	 * Execute the specified command with the specified parameters and return
	 * the result to the caller. The format of the command, parameters and 
	 * the result are determined by the acutal implementation.
	 */
    string exec(in string aCommand, in string aParameters);
	/*
	 * Query the status of the specified command with the specified parameters
	 * and return the result to the caller. The format of the command,
	 * parameters and the result are determined by the implementation.
	 */
    string query(in string aCommand, in string aParameters);
};

%{ C++
//	{3A449110-66FD-11d4-9528-0020183BF181} - 
#define NS_COMMANDHANDLER_CID \
{ 0x3a449110, 0x66fd, 0x11d4, { 0x95, 0x28, 0x0, 0x20, 0x18, 0x3b, 0xf1, 0x81 } }
#define NS_COMMANDHANDLER_CONTRACTID \
"@mozilla.org/embedding/browser/nsCommandHandler;1"
%}

