/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  vim: sw=2 ts=2 et lcs=trail\:.,tab\:>~ :
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable,function, uuid(0bfee0c4-2c24-400e-b18e-b5bb41a032c8)]
interface mozIStorageCompletionCallback : nsISupports {
  /**
   * Indicates that the event this callback was passed in for has completed.
   */
  void complete();
};
