/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsIDOMMozActivityHandlerDescription;
interface nsIDOMDOMRequest;

/**
  * This interface is implemented by the Navigator object.
  */
[scriptable, uuid(e7cb7d2f-11d2-4783-a8b0-bddabb4a5c03)]
interface nsIDOMMozNavigatorActivities : nsISupports
{
    nsIDOMDOMRequest mozRegisterActivityHandler(in nsIDOMMozActivityHandlerDescription description);
    void mozUnregisterActivityHandler(in nsIDOMMozActivityHandlerDescription description);

    bool mozIsActivityHandlerRegistered(in nsIDOMMozActivityHandlerDescription description);
};
