/* THIS FILE IS AUTOGENERATED - DO NOT EDIT */

#ifndef mozilla_dom_CFStateChangeEventBinding_h__
#define mozilla_dom_CFStateChangeEventBinding_h__

#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/DOMJSClass.h"
#include "mozilla/dom/DOMJSProxyHandler.h"

namespace mozilla {
namespace dom {

struct CFStateChangeEventDict : public MainThreadDictionaryBase
{
  int16_t mAction;
  nsString mNumber;
  int16_t mReason;
  int16_t mServiceClass;
  bool mSuccess;
  int16_t mTimeSeconds;

private:
  static bool initedIds;
  static jsid action_id;
  static jsid number_id;
  static jsid reason_id;
  static jsid serviceClass_id;
  static jsid success_id;
  static jsid timeSeconds_id;

  CFStateChangeEventDict(const CFStateChangeEventDict&) MOZ_DELETE;

public:
  inline CFStateChangeEventDict()
  {
  }

  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val);

  bool
  Init(const nsAString& aJSON);

  bool
  ToObject(JSContext* cx, JS::Handle<JSObject*> parentObject, JS::MutableHandle<JS::Value> rval) const;

  void
  TraceDictionary(JSTracer* trc);

private:
  static bool
  InitIds(JSContext* cx);
};
struct CFStateChangeEventDictInitializer : public CFStateChangeEventDict
{
  inline CFStateChangeEventDictInitializer()
  {
    // Safe to pass a null context if we pass a null value
    Init(nullptr, JS::NullHandleValue);
  }
};


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_CFStateChangeEventBinding_h__
